"""
flask_alembic
-------------

.. autoclass:: flask_alembic.Alembic
    :members:
    :member-order: bysource

.. automodule:: flask_alembic.cli
    :members:
"""
from flask_alembic.extension import Alembic
